from collections.abc import Iterable,Iterator
from typing import List

def chunking(sequence:Iterable,chunk_len:int)->Iterator:
    '''
    将输入序列切分成若干个块
    例如：sequence=[1,2,3,4,5,6,7,8,9,10],chunk_len=3
    返回：[[1,2,3],[4,5,6],[7,8,9],[10]]
    '''
    chunk,ct=[],0
    for item in sequence:
        chunk.append(item)
        ct+=1
        if ct%chunk_len==0:
            yield chunk
            chunk=[]
    if chunk:
        yield chunk


def lower_boundary(nums: List[object], target: object) -> object:
    '''
    在递增序列nums中查找target的下边界，即第一个大于等于target的数，并返回其下标
    例如：nums=[5,7,7,8,8,10],target=7
    返回：1

    '''
    l=0
    r=len(nums)
    while l<r:
        m=(l+r)//2
        if nums[m]<target:
            l=m+1  
        else:
            r=m
    return l

def upper_boundary(nums: List[object], target: object) -> object:
    '''
    在递增序列nums中查找target的上边界，即第一个大于target的数，并返回其下标
    例如：nums=[5,7,7,8,8,10],target=7
    返回：3

    '''
    l=0
    r=len(nums)
    while l<r:
        m=(l+r)//2
        if nums[m]<=target:
            l=m+1
        else:
            r=m
    return l
    



if __name__=='__main__':

    a=[5,7,7,8,8,10]
    print(lower_boundary(a,7))


from collections import deque

class MaxQueue:
    '''
    在队列的基础上增加max_value函数，用于计算当前队列中的最大值
    '''

    def __init__(self):
        self.queue=deque()
        self.candidates=deque()

    def max_value(self) -> int:
        '''
        返回当前队列中的最大值
        '''
        if not self.queue:
            return -1
        else:
            return self.candidates[0]


    def push(self, value: int) -> None:
        '''
        将value加入到队尾
        '''
        self.queue.append(value)
        while self.candidates and self.candidates[-1]<value:
            self.candidates.pop()
        self.candidates.append(value)

    def pop(self) -> int:
        '''
        删除队首元素，并将其返回
        '''
        if not self.queue:
            return -1
        else:
            item=self.queue.popleft()
            if item==self.candidates[0]:
                self.candidates.popleft()
            return item
            




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @File     : setup
# @Author   : LiuYan
# @Time     : 2021/4/16 10:07

import setuptools

with open('./huailib/README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name='huailib',
    version='0.0.2',
    author='huai guo',
    author_email='guohuai@csu.edu.cn',
    description='a helpful python library',
    long_description=long_description,
    url='https://github.com/wsmgh/hpl.git',
    packages=setuptools.find_packages(),

)

